@@ -12,7 +12,7 @@ import (
 	"github.com/rclone/rclone/lib/terminal"
 )
 
-func (b *bisyncRun) indentf(tag, file, format string, args ...interface{}) {
+func (b *bisyncRun) indentf(tag, file, format string, args ...any) {
 	b.indent(tag, file, fmt.Sprintf(format, args...))
 }
 
@@ -66,7 +66,8 @@ func quotePath(path string) string {
 	return escapePath(path, true)
 }
 
-var Colors bool // Colors controls whether terminal colors are enabled
+// Colors controls whether terminal colors are enabled
+var Colors bool
 
 // Color handles terminal colors for bisync
 func Color(style string, s string) string {
@@ -77,6 +78,15 @@ func Color(style string, s string) string {
 	return style + s + terminal.Reset
 }
 
+// ColorX handles terminal colors for bisync
+func ColorX(style string, s string) string {
+	if !Colors {
+		return s
+	}
+	terminal.Start()
+	return style + s + terminal.Reset
+}
+
 func encode(s string) string {
 	return encoder.OS.ToStandardPath(encoder.OS.FromStandardPath(s))
 }
