@@ -62,7 +62,7 @@ func rcList(ctx context.Context, in rc.Params) (out rc.Params, err error) {
 	if rc.NotErrParamNotFound(err) {
 		return nil, err
 	}
-	var list = []*ListJSONItem{}
+	list := []*ListJSONItem{}
 	err = ListJSON(ctx, f, remote, &opt, func(item *ListJSONItem) error {
 		list = append(list, item)
 		return nil
@@ -193,7 +193,7 @@ func rcMoveOrCopyFile(ctx context.Context, in rc.Params, cp bool) (out rc.Params
 	if err != nil {
 		return nil, err
 	}
-	return nil, moveOrCopyFile(ctx, dstFs, srcFs, dstRemote, srcRemote, cp)
+	return nil, moveOrCopyFile(ctx, dstFs, srcFs, dstRemote, srcRemote, cp, false)
 }
 
 func init() {
@@ -289,7 +289,6 @@ func rcSingleCommand(ctx context.Context, in rc.Params, name string, noRemote bo
 
 		var request *http.Request
 		request, err := in.GetHTTPRequest()
-
 		if err != nil {
 			return nil, err
 		}
@@ -629,20 +628,19 @@ func rcBackend(ctx context.Context, in rc.Params) (out rc.Params, err error) {
 	if err != nil {
 		return nil, err
 	}
-	var opt = map[string]string{}
+	opt := map[string]string{}
 	err = in.GetStructMissingOK("opt", &opt)
 	if err != nil {
 		return nil, err
 	}
-	var arg = []string{}
+	arg := []string{}
 	err = in.GetStructMissingOK("arg", &arg)
 	if err != nil {
 		return nil, err
 	}
-	result, err := doCommand(context.Background(), command, arg, opt)
+	result, err := doCommand(ctx, command, arg, opt)
 	if err != nil {
 		return nil, fmt.Errorf("command %q failed: %w", command, err)
-
 	}
 	out = make(rc.Params)
 	out["result"] = result
@@ -685,7 +683,6 @@ func rcDu(ctx context.Context, in rc.Params) (out rc.Params, err error) {
 	dir, err := in.GetString("dir")
 	if rc.IsErrParamNotFound(err) {
 		dir = config.GetCacheDir()
-
 	} else if err != nil {
 		return nil, err
 	}
@@ -819,10 +816,8 @@ func rcCheck(ctx context.Context, in rc.Params) (out rc.Params, err error) {
 		if srcFs != nil {
 			return nil, rc.NewErrParamInvalid(errors.New("only supply dstFs when using checkFileHash"))
 		}
-	} else {
-		if srcFs == nil {
-			return nil, rc.NewErrParamInvalid(errors.New("need srcFs parameter when not using checkFileHash"))
-		}
+	} else if srcFs == nil {
+		return nil, rc.NewErrParamInvalid(errors.New("need srcFs parameter when not using checkFileHash"))
 	}
 
 	oneway, _ := in.GetBool("oneway")
@@ -858,13 +853,13 @@ func rcCheck(ctx context.Context, in rc.Params) (out rc.Params, err error) {
 
 	if checkFileHash != "" {
 		out["hashType"] = checkFileHashType.String()
-		err = CheckSum(context.Background(), dstFs, checkFileFs, checkFileRemote, checkFileHashType, opt, download)
+		err = CheckSum(ctx, dstFs, checkFileFs, checkFileRemote, checkFileHashType, opt, download)
 	} else {
 		if download {
-			err = CheckDownload(context.Background(), opt)
+			err = CheckDownload(ctx, opt)
 		} else {
 			out["hashType"] = srcFs.Hashes().Overlap(dstFs.Hashes()).GetOne().String()
-			err = Check(context.Background(), opt)
+			err = Check(ctx, opt)
 		}
 	}
 	if err != nil {
